/**********************************************************************
 *
 * nullproto.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

#include "nullproto.h"
#include "colservrconfig.h"
#include <assert.h>

#include "filter.h"
#include "browsefilter.h"
#include "mgqueryfilter.h"
#include "mgppqueryfilter.h"
#include "infodbclass.h"
#include "mggdbmsource.h"
#include "mgsearch.h"
#include "mgppsearch.h"
#include "fileutil.h"


nullproto::~nullproto() {
  collectservermapclass::iterator here = cservers.begin();
  collectservermapclass::iterator end = cservers.end();

  while (here != end) {
    if ((*here).second.c != NULL) {
      delete (*here).second.c;
    }
    here ++;
  }
  cservers.clear();
}

// add_collection sets up the collectionserver and calls
// add_collectserver
void nullproto::add_collection (const text_t &collection, void *recpt, 
				const text_t &gsdlhome, const text_t &gdbmhome) {
      
  // read config file to see if built with mg or mgpp
  // -- we can rely on the collection (and therefore the build.cfg)
  // being here since this is the null protocol - a nicer way to 
  // do this would be preferable though - Stefan.
  text_t buildtype = "mg"; // mg is default
  // (for now we'll just ignore mgpp if on windows)
#ifndef __WIN32__
  text_tarray cfgline;
  text_t key;
  text_t build_cfg = filename_cat(gsdlhome, "collect", collection, "index", "build.cfg");
  char *build_cfgc = build_cfg.getcstr();
  ifstream confin(build_cfgc);

  if (confin) {
    while (read_cfg_line(confin, cfgline) >= 0) {
      if (cfgline.size() == 2) {
	key = cfgline[0];
	cfgline.erase(cfgline.begin());
	if (key == "buildtype") {
	  buildtype = cfgline[0];
	  break;
	}
      }
    }
    confin.close();
  }
  delete build_cfgc;
#endif

  collectserver *cserver = new collectserver();
  gdbmclass *gdbmhandler = new gdbmclass();

  // add a null filter
  filterclass *filter = new filterclass ();
  cserver->add_filter (filter);
      
  // add a browse filter
  browsefilterclass *browsefilter = new browsefilterclass();
  browsefilter->set_gdbmptr (gdbmhandler);

  cserver->add_filter (browsefilter);  
 
  if (buildtype == "mg") {
    mgsearchclass *mgsearch = new mgsearchclass();
 
    // add a query filter
    mgqueryfilterclass *queryfilter = new mgqueryfilterclass();
    queryfilter->set_gdbmptr (gdbmhandler);
    queryfilter->set_mgsearchptr (mgsearch);
    cserver->add_filter (queryfilter);
    
    // add a mg and gdbm source
    mggdbmsourceclass *mggdbmsource = new mggdbmsourceclass ();
    mggdbmsource->set_gdbmptr (gdbmhandler);
    mggdbmsource->set_mgsearchptr (mgsearch);
    cserver->add_source (mggdbmsource);
  }

#ifndef __WIN32__

  else if (buildtype == "mgpp") {
      
    mgppsearchclass *mgsearch = new mgppsearchclass();

    // add a query filter
    mgppqueryfilterclass *queryfilter = new mgppqueryfilterclass();
    queryfilter->set_gdbmptr (gdbmhandler);
    queryfilter->set_mgsearchptr (mgsearch);
    cserver->add_filter (queryfilter);
      
    // add a mg and gdbm source
    mggdbmsourceclass *mggdbmsource = new mggdbmsourceclass ();
    mggdbmsource->set_gdbmptr (gdbmhandler);
    mggdbmsource->set_mgsearchptr (mgsearch);
    cserver->add_source (mggdbmsource);
    
  }
#endif    

  // inform collection server and everything it contains about its
  // collection name
  cserver->configure ("collection", collection);
  
  // configure receptionist's collectinfo structure 
  text_tarray colinfo;
  colinfo.push_back (collection);
  colinfo.push_back (gsdlhome);
  colinfo.push_back (gdbmhome);
  receptionist *rcp = (receptionist *)recpt;
  rcp->configure ("collectinfo", colinfo);
  
  add_collectserver (cserver);
}

// remove_collection deletes the collection server of collection.
// This only needs to be called if a collectionserver is to be
// removed while the library is running. The destructor function
// cleans up all collectservers when the program exits.
void nullproto::remove_collection (const text_t &collection, ostream &logout) {
  collectservermapclass::iterator here = cservers.begin();
  collectservermapclass::iterator end = cservers.end();

  while (here != end) {
    if ((*here).second.c != NULL && (*here).first == collection) {
      delete (*here).second.c;
      cservers.erase (here);
      return;
    }
    here ++;
  }
  outconvertclass text_t2ascii;
  logout << text_t2ascii << "nullproto::remove_collection: failed to remove collectserver for "
	 << collection << "\n";
}

// this configure will configure each of the collection servers
void nullproto::configure (const text_t &key, const text_tarray &cfgline) {
  // the naming of the collection should not be done here,
  // it should be done just after the collection server has been 
  // created
  if (key == "collection" || key == "collectdir") return;

  collectservermapclass::iterator here = cservers.begin();
  collectservermapclass::iterator end = cservers.end();

  while (here != end) {
    assert ((*here).second.c != NULL);
    if ((*here).second.c != NULL) {
      if (key == "collectinfo") {
	if ((*here).first == cfgline[0]) {
	  (*here).second.c->configure ("gsdlhome", cfgline[1]);
	  (*here).second.c->configure ("gdbmhome", cfgline[2]);
	}
      } else {
	(*here).second.c->configure (key, cfgline);
      }
    }

    here++;
  }
}

// this init will configure and init each of the collection servers
bool nullproto::init (ostream &logout) {
  collectservermapclass::iterator here = cservers.begin();
  collectservermapclass::iterator end = cservers.end();

  while (here != end) {
    assert ((*here).second.c != NULL);
    if ((*here).second.c != NULL) {
      const colservrconf &configinfo = (*here).second.c->get_configinfo ();
      bool failed = false;

      // configure this collection server

      // note that we read build.cfg before collect.cfg so that the indexmaps
      // are available to decode defaultindex, defaultsubcollection, and
      // defaultlanguage
      if (!build_cfg_read (*((*here).second.c), configinfo.gsdlhome,
			   configinfo.collection)) {
	outconvertclass text_t2ascii;
	logout << text_t2ascii 
	       << "Warning: couldn't read build.cfg file for collection \"" //****
	       << configinfo.collection << "\", gsdlhome=\""
	       << configinfo.gsdlhome << "\"\n";
	failed = true; 
      }

      if (!collect_cfg_read (*((*here).second.c), configinfo.gsdlhome,
			     configinfo.collection)) {
	outconvertclass text_t2ascii;
	logout << text_t2ascii 
	       << "Warning: couldn't read collect.cfg file for collection \""
	       << configinfo.collection << "\", gsdlhome=\"" 
	       << configinfo.gsdlhome << "\"\n";
	failed = true;
      }

      // don't bother initializing if one of the config files
      // wasn't read correctly
      if (!failed) {
	if (!(*here).second.c->init (logout)) return false;
      }
    }
    here++;
  }

  return true;
}

text_t nullproto::get_protocol_name () {
  return "nullproto";
}


void nullproto::get_collection_list (text_tarray &collist, comerror_t &err, 
				     ostream &/*logout*/) {
  collist.erase(collist.begin(),collist.end());
  err = noError;

  collectservermapclass::iterator here = cservers.begin();
  collectservermapclass::iterator end = cservers.end();
  while (here != end) {
    assert ((*here).second.c != NULL);
    if ((*here).second.c != NULL) {
      collist.push_back ((*here).second.c->get_collection_name());
    }
    here++;
  }
}

void nullproto::has_collection (const text_t &collection, bool &hascollection, 
				comerror_t &err, ostream &/*logout*/) {
  hascollection = (cservers.getcollectserver(collection) != NULL);
  err = noError;
}

void nullproto::ping (const text_t &collection, bool &wassuccess, 
		      comerror_t &err, ostream &/*logout*/) {
  wassuccess = (cservers.getcollectserver(collection) != NULL);
  err = noError;
}

void nullproto::get_collectinfo (const text_t &collection, 
				 ColInfoResponse_t &collectinfo,
				 comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->get_collectinfo (collectinfo, err, logout);
  else err = protocolError;
}


void nullproto::get_filterinfo (const text_t &collection,
				InfoFiltersResponse_t &response,
				comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->get_filterinfo (response, err, logout);
  else err = protocolError;
}

void nullproto::get_filteroptions (const text_t &collection,
				   const InfoFilterOptionsRequest_t &request,
				   InfoFilterOptionsResponse_t &response, 
				   comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->get_filteroptions (request, response, err, logout);
  else err = protocolError;
}

void nullproto::filter (const text_t &collection,
			FilterRequest_t &request,
			FilterResponse_t &response,
			comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->filter (request, response, err, logout);
  else err = protocolError;
}

void nullproto::get_document (const text_t &collection,
			      const DocumentRequest_t &request,
			      DocumentResponse_t &response,
			      comerror_t &err, ostream &logout) {
  collectserver *cserver = cservers.getcollectserver (collection);
  if (cserver != NULL) cserver->get_document (request, response, err, logout);
  else err = protocolError;
}
